--------------------------------------------------------------------------------
-- Title   : clock sources
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : sub frequecy clocks generated from the main clock
--------------------------------------------------------------------------------
-- File     : clk_sources.vhd
-- Revision : x.x.x
-- Created  : October 30, 2012
-- Updated  : March 11, 2013
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity clk_sources is
  port (
    p_clk60MHz  : in  std_logic; -- clock 60 MHz
    p_reset     : in  std_logic; -- reset input
    p_clk1kHz   : out std_logic; -- clock 0.1 Hz
    p_clk500mHz : out std_logic  -- clock 0.5 Hz
  );
end entity clk_sources;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_clk_sources of clk_sources is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_lfsr   : std_logic_vector (15 downto 0) := "0101011011101011";
  constant c_cntmax : unsigned (8 downto 0) := "111110011"; -- /500
  constant c_cntH   : unsigned (7 downto 0) := "11001000";

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_lfsr      : std_logic_vector (15 downto 0);
  signal s_lfsr_lsb  : std_logic;
  signal s_cnt       : unsigned (8 downto 0); -- counter
  signal s_clk1kHz   : std_logic;
  signal s_clk500mHz : std_logic;

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- 1 kHz clock
  process (p_clk60MHz, p_reset)
  begin
    if (p_reset = '1') then
      s_lfsr <= (OTHERS => '0');
      s_clk1kHz <= '0';
    elsif (rising_edge(p_clk60MHz)) then
      if(s_lfsr = c_lfsr) then
        s_lfsr <= (OTHERS => '0');
        s_clk1kHz <= '1';
      else
        s_lfsr <= s_lfsr(14 downto 0) & s_lfsr_lsb;
        s_clk1kHz <= '0';
      end if;
    end if;
  end process;

  -- 0.5 Hz clock
  process (p_reset, s_clk1kHz)
  begin
    if (p_reset = '1') then
      s_cnt <= (OTHERS => '0');
    elsif (rising_edge(s_clk1kHz)) then
      if (s_cnt < c_cntmax) then
        s_cnt <= s_cnt + 1;
      else
        s_cnt <= (OTHERS => '0');
      end if;
    end if;
  end process;

  process (p_reset, s_clk1kHz)
  begin
    if (p_reset = '1') then
      s_clk500mHz <= '1';
    elsif (rising_edge(s_clk1kHz)) then
      if (s_cnt < c_cntH) then
        s_clk500mHz <= '1';
      else
        s_clk500mHz <= '0';
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  s_lfsr_lsb  <= s_lfsr(15) xnor s_lfsr(14) xnor s_lfsr(12) xnor s_lfsr(3);
  p_clk1kHz <= s_clk1kHz;
  p_clk500mHz <= s_clk500mHz;

end architecture rtl_clk_sources;
